@@ -1,32 +1,6 @@
-/*
-  Copyright 2010 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <iomanip>
 
@@ -59,6 +33,8 @@ public:
     virtual bool seek_subimage(int subimage, int miplevel) override;
     virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
                                       void* data) override;
+    virtual bool read_native_scanlines(int subimage, int miplevel, int ybegin,
+                                       int yend, int z, void* data) override;
 
 private:
     int m_subimage;
@@ -66,19 +42,18 @@ private:
     dpx::Reader m_dpx;
     std::vector<unsigned char> m_userBuf;
     bool m_rawcolor;
-    unsigned char* m_dataPtr = nullptr;
+    std::vector<unsigned char> m_decodebuf;  // temporary decode buffer
 
     /// Reset everything to initial state
     ///
     void init()
     {
+        m_subimage = -1;
         if (m_stream) {
             m_stream->Close();
             delete m_stream;
             m_stream = nullptr;
         }
-        delete m_dataPtr;
-        m_dataPtr = nullptr;
         m_userBuf.clear();
         m_rawcolor = false;
     }
@@ -150,13 +125,13 @@ DPXInput::open(const std::string& name, ImageSpec& newspec)
     // open the image
     m_stream = new InStream();
     if (!m_stream->Open(name.c_str())) {
-        error("Could not open file \"%s\"", name.c_str());
+        errorf("Could not open file \"%s\"", name);
         return false;
     }
 
     m_dpx.SetInStream(m_stream);
     if (!m_dpx.ReadHeader()) {
-        error("Could not read header");
+        errorf("Could not read header");
         close();
         return false;
     }
@@ -189,6 +164,8 @@ DPXInput::seek_subimage(int subimage, int miplevel)
 {
     if (miplevel != 0)
         return false;
+    if (subimage == m_subimage)
+        return true;
     if (subimage < 0 || subimage >= m_dpx.header.ImageElementCount())
         return false;
 
@@ -211,7 +188,7 @@ DPXInput::seek_subimage(int subimage, int miplevel)
         break;
     case dpx::kFloat: typedesc = TypeDesc::FLOAT; break;
     case dpx::kDouble: typedesc = TypeDesc::DOUBLE; break;
-    default: error("Invalid component data size"); return false;
+    default: errorf("Invalid component data size"); return false;
     }
     m_spec = ImageSpec(m_dpx.header.Width(), m_dpx.header.Height(),
                        m_dpx.header.ImageElementComponentCount(subimage),
@@ -240,11 +217,7 @@ DPXInput::seek_subimage(int subimage, int miplevel)
         m_spec.channelnames.emplace_back("A");
         m_spec.alpha_channel = 0;
         break;
-    case dpx::kLuma:
-        // FIXME: do we treat this as intensity or do we use Y' as per
-        // convention to differentiate it from linear luminance?
-        m_spec.channelnames.emplace_back("Y'");
-        break;
+    case dpx::kLuma: m_spec.channelnames.emplace_back("Y"); break;
     case dpx::kDepth:
         m_spec.channelnames.emplace_back("Z");
         m_spec.z_channel = 0;
@@ -562,16 +535,10 @@ DPXInput::seek_subimage(int subimage, int miplevel)
                          TypeDesc(TypeDesc::UCHAR, m_dpx.header.UserSize()),
                          &m_userBuf[0]);
 
-    dpx::Block block(0, 0, m_dpx.header.Width() - 1, 0);
-    int bufsize = dpx::QueryRGBBufferSize(m_dpx.header, subimage, block);
-    if (bufsize == 0 && !m_rawcolor) {
-        error("Unable to deliver RGB data from source data");
-        return false;
-    } else if (!m_rawcolor && bufsize > 0)
-        m_dataPtr = new unsigned char[bufsize];
-    else
-        // no need to allocate another buffer
-        m_dataPtr = NULL;
+    // All of the 1-channel encoding options also behave like "rawcolor",
+    // not needing color space transformations.
+    if (m_spec.nchannels == 1)
+        m_rawcolor = true;
 
     return true;
 }
@@ -590,25 +557,39 @@ DPXInput::close()
 bool
 DPXInput::read_native_scanline(int subimage, int miplevel, int y, int z,
                                void* data)
+{
+    return read_native_scanlines(subimage, miplevel, y, y + 1, z, data);
+}
+
+
+
+bool
+DPXInput::read_native_scanlines(int subimage, int miplevel, int ybegin,
+                                int yend, int z, void* data)
 {
     lock_guard lock(m_mutex);
     if (!seek_subimage(subimage, miplevel))
         return false;
 
-    dpx::Block block(0, y - m_spec.y, m_dpx.header.Width() - 1, y - m_spec.y);
+    dpx::Block block(0, ybegin - m_spec.y, m_dpx.header.Width() - 1,
+                     yend - 1 - m_spec.y);
 
     if (m_rawcolor) {
         // fast path - just read the scanline in
-        if (!m_dpx.ReadBlock(m_subimage, (unsigned char*)data, block))
+        if (!m_dpx.ReadBlock(subimage, (unsigned char*)data, block))
             return false;
     } else {
         // read the scanline and convert to RGB
-        void* ptr = m_dataPtr == NULL ? data : (void*)m_dataPtr;
+        unsigned char* ptr = (unsigned char*)data;
+        int bufsize = dpx::QueryRGBBufferSize(m_dpx.header, subimage, block);
+        if (bufsize > 0) {
+            m_decodebuf.resize(bufsize);
+            ptr = m_decodebuf.data();
+        }
 
-        if (!m_dpx.ReadBlock(m_subimage, (unsigned char*)ptr, block))
+        if (!m_dpx.ReadBlock(subimage, ptr, block))
             return false;
-
-        if (!dpx::ConvertToRGB(m_dpx.header, m_subimage, ptr, data, block))
+        if (!dpx::ConvertToRGB(m_dpx.header, subimage, ptr, data, block))
             return false;
     }
 
