@@ -767,7 +767,7 @@ void dpx::GenericHeader::SetCreationTimeDate(const long sec)
 	struct tm *tm_time;
 	char str[32];
 	
-#ifdef WIN32
+#ifdef _WIN32
 	_tzset();
 #endif
 
@@ -783,7 +783,7 @@ void dpx::GenericHeader::SetSourceTimeDate(const long sec)
 	struct tm *tm_time;
 	char str[32];
 	
-#ifdef WIN32
+#ifdef _WIN32
 	_tzset();
 #endif
 
