@@ -1,32 +1,6 @@
-/*
-  Copyright 2010 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cmath>
 #include <cstdio>
@@ -101,7 +75,7 @@ private:
     void init()
     {
         m_name.clear();
-        ASSERT(!m_input);
+        OIIO_ASSERT(!m_input);
         m_subimage   = -1;
         m_nsubimages = 0;
         m_layers.clear();
@@ -154,7 +128,7 @@ template<typename T>
 inline int
 blocksize(FieldRes::Ptr& f)
 {
-    ASSERT(f && "taking blocksize of null ptr");
+    OIIO_DASSERT(f && "taking blocksize of null ptr");
     typename SparseField<T>::Ptr sf(field_dynamic_cast<SparseField<T>>(f));
     if (sf)
         return sf->blockSize();
@@ -274,8 +248,8 @@ Field3DInput::read_one_layer(FieldRes::Ptr field, layerrecord& lay,
         lay.spec.tile_height = lay.spec.height;
         lay.spec.tile_depth  = lay.spec.depth;
     }
-    ASSERT(lay.spec.tile_width > 0 && lay.spec.tile_height > 0
-           && lay.spec.tile_depth > 0);
+    OIIO_ASSERT(lay.spec.tile_width > 0 && lay.spec.tile_height > 0
+                && lay.spec.tile_depth > 0);
 
     lay.spec.attribute("ImageDescription", lay.unique_name);
     lay.spec.attribute("oiio:subimagename", lay.unique_name);
@@ -328,7 +302,7 @@ Field3DInput::read_layers(TypeDesc datatype)
             else if (field_dynamic_cast<SparseField<Data_T>>(*i))
                 lay.fieldtype = f3dpvt::Sparse;
             else
-                ASSERT(0 && "unknown field type");
+                OIIO_ASSERT(0 && "unknown field type");
             read_one_layer(*i, lay, datatype, layernum);
         }
     }
@@ -350,7 +324,7 @@ Field3DInput::read_layers(TypeDesc datatype)
             else if (field_dynamic_cast<MACField<VecData_T>>(*i))
                 lay.fieldtype = f3dpvt::MAC;
             else
-                ASSERT(0 && "unknown field type");
+                OIIO_ASSERT(0 && "unknown field type");
             lay.vecfield = true;
             read_one_layer(*i, lay, datatype, layernum);
         }
