@@ -1,32 +1,6 @@
-/*
-  Copyright 2010 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #pragma once
 
@@ -135,6 +109,13 @@ public:
     /// may be either a color space name or a role.
     OIIO::TypeDesc getColorSpaceDataType(string_view name, int* bits) const;
 
+    /// Retrieve the full list of known color space names, as a vector
+    /// of strings.
+    std::vector<std::string> getColorSpaceNames() const;
+
+    /// Get the name of the color space family of the named color space,
+    /// or NULL if none could be identified.
+    const char* getColorSpaceFamilyByName(string_view name) const;
 
     /// Get the number of Looks defined in this configuration
     int getNumLooks() const;
@@ -142,6 +123,9 @@ public:
     /// Query the name of the specified Look.
     const char* getLookNameByIndex(int index) const;
 
+    /// Retrieve the full list of known look names, as a vector of strings.
+    std::vector<std::string> getLookNames() const;
+
     /// Given the specified input and output ColorSpace, request a handle to
     /// a ColorProcessor.  It is possible that this will return an empty
     /// handle, if the inputColorSpace doesnt exist, the outputColorSpace
@@ -192,17 +176,29 @@ public:
     /// Query the name of the specified display.
     const char* getDisplayNameByIndex(int index) const;
 
-    /// Get the number of views for a given display defined in this configuration
-    int getNumViews(string_view display) const;
+    /// Retrieve the full list of known display names, as a vector of
+    /// strings.
+    std::vector<std::string> getDisplayNames() const;
+
+    /// Get the name of the default display.
+    const char* getDefaultDisplayName() const;
+
+    /// Get the number of views for a given display defined in this
+    /// configuration. If the display is empty or not specified, the default
+    /// display will be used.
+    int getNumViews(string_view display = "") const;
 
     /// Query the name of the specified view for the specified display
     const char* getViewNameByIndex(string_view display, int index) const;
 
-    /// Query the name of the default display
-    const char* getDefaultDisplayName() const;
+    /// Retrieve the full list of known view names for the display, as a
+    /// vector of strings. If the display is empty or not specified, the
+    /// default display will be used.
+    std::vector<std::string> getViewNames(string_view display = "") const;
 
-    /// Query the name of the default view for the specified display
-    const char* getDefaultViewName(string_view display) const;
+    /// Query the name of the default view for the specified display. If the
+    /// display is empty or not specified, the default display will be used.
+    const char* getDefaultViewName(string_view display = "") const;
 
     /// Construct a processor to transform from the given color space
     /// to the color space of the given display and view. You may optionally
