@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <algorithm>
 #include <cassert>
@@ -151,7 +125,7 @@ JpgInput::jpegerror(my_error_ptr myerr, bool fatal)
     // Send the error message to the ImageInput
     char errbuf[JMSG_LENGTH_MAX];
     (*m_cinfo.err->format_message)((j_common_ptr)&m_cinfo, errbuf);
-    error("JPEG error: %s (\"%s\")", errbuf, filename().c_str());
+    errorf("JPEG error: %s (\"%s\")", errbuf, filename());
 
     // Shut it down and clean it up
     if (fatal) {
@@ -164,16 +138,21 @@ JpgInput::jpegerror(my_error_ptr myerr, bool fatal)
 
 
 bool
-JpgInput::valid_file(const std::string& filename) const
+JpgInput::valid_file(const std::string& filename, Filesystem::IOProxy* io) const
 {
-    FILE* fd = Filesystem::fopen(filename, "rb");
-    if (!fd)
-        return false;
-
     // Check magic number to assure this is a JPEG file
     uint8_t magic[2] = { 0, 0 };
-    bool ok          = (fread(magic, sizeof(magic), 1, fd) == 1);
-    fclose(fd);
+    bool ok          = true;
+
+    if (io) {
+        ok = (io->pread(magic, sizeof(magic), 0) == sizeof(magic));
+    } else {
+        FILE* fd = Filesystem::fopen(filename, "rb");
+        if (!fd)
+            return false;
+        ok = (fread(magic, sizeof(magic), 1, fd) == 1);
+        fclose(fd);
+    }
 
     if (magic[0] != JPEG_MAGIC1 || magic[1] != JPEG_MAGIC2) {
         ok = false;
@@ -187,8 +166,10 @@ bool
 JpgInput::open(const std::string& name, ImageSpec& newspec,
                const ImageSpec& config)
 {
-    const ParamValue* p = config.find_attribute("_jpeg:raw", TypeInt);
-    m_raw               = p && *(int*)p->data();
+    auto p = config.find_attribute("_jpeg:raw", TypeInt);
+    m_raw  = p && *(int*)p->data();
+    p      = config.find_attribute("oiio:ioproxy", TypeDesc::PTR);
+    m_io   = p ? p->get<Filesystem::IOProxy*>() : nullptr;
     return open(name, newspec);
 }
 
@@ -197,28 +178,39 @@ JpgInput::open(const std::string& name, ImageSpec& newspec,
 bool
 JpgInput::open(const std::string& name, ImageSpec& newspec)
 {
-    // Check that file exists and can be opened
     m_filename = name;
-    m_fd       = Filesystem::fopen(name, "rb");
-    if (m_fd == NULL) {
-        error("Could not open file \"%s\"", name.c_str());
+
+    if (m_io) {
+        // If an IOProxy was passed, it had better be a File or a
+        // MemReader, that's all we know how to use with jpeg.
+        std::string proxytype = m_io->proxytype();
+        if (proxytype != "file" && proxytype != "memreader") {
+            errorf("JPEG reader can't handle proxy type %s", proxytype);
+            return false;
+        }
+    } else {
+        // If no proxy was supplied, create a file reader
+        m_io = new Filesystem::IOFile(name, Filesystem::IOProxy::Mode::Read);
+        m_local_io.reset(m_io);
+    }
+    if (!m_io || m_io->mode() != Filesystem::IOProxy::Mode::Read) {
+        errorf("Could not open file \"%s\"", name);
         return false;
     }
 
     // Check magic number to assure this is a JPEG file
     uint8_t magic[2] = { 0, 0 };
-    if (fread(magic, sizeof(magic), 1, m_fd) != 1) {
-        error("Empty file \"%s\"", name.c_str());
+    if (m_io->pread(magic, sizeof(magic), 0) != sizeof(magic)) {
+        errorf("Empty file \"%s\"", name);
         close_file();
         return false;
     }
 
-    rewind(m_fd);
     if (magic[0] != JPEG_MAGIC1 || magic[1] != JPEG_MAGIC2) {
         close_file();
-        error(
+        errorf(
             "\"%s\" is not a JPEG file, magic number doesn't match (was 0x%x%x)",
-            name.c_str(), int(magic[0]), int(magic[1]));
+            name, int(magic[0]), int(magic[1]));
         return false;
     }
 
@@ -235,8 +227,18 @@ JpgInput::open(const std::string& name, ImageSpec& newspec)
         return false;
     }
 
-    jpeg_create_decompress(&m_cinfo);  // initialize decompressor
-    jpeg_stdio_src(&m_cinfo, m_fd);    // specify the data source
+    // initialize decompressor
+    jpeg_create_decompress(&m_cinfo);
+    m_decomp_create = true;
+    // specify the data source
+    if (!strcmp(m_io->proxytype(), "file")) {
+        auto fd = ((Filesystem::IOFile*)m_io)->handle();
+        jpeg_stdio_src(&m_cinfo, fd);
+    } else {
+        auto buffer = ((Filesystem::IOMemReader*)m_io)->buffer();
+        jpeg_mem_src(&m_cinfo, const_cast<unsigned char*>(buffer.data()),
+                     buffer.size());
+    }
 
     // Request saving of EXIF and other special tags for later spelunking
     for (int mark = 0; mark < 16; ++mark)
@@ -245,7 +247,7 @@ JpgInput::open(const std::string& name, ImageSpec& newspec)
 
     // read the file parameters
     if (jpeg_read_header(&m_cinfo, FALSE) != JPEG_HEADER_OK || m_fatalerr) {
-        error("Bad JPEG header for \"%s\"", filename().c_str());
+        errorf("Bad JPEG header for \"%s\"", filename());
         return false;
     }
 
@@ -443,7 +445,7 @@ JpgInput::read_native_scanline(int subimage, int miplevel, int y, int z,
         if (!close() || !open(m_filename, dummyspec)
             || !seek_subimage(subimage, 0))
             return false;  // Somehow, the re-open failed
-        assert(m_next_scanline == 0 && current_subimage() == subimage);
+        OIIO_DASSERT(m_next_scanline == 0 && current_subimage() == subimage);
     }
 
     // Set up our custom error handler
@@ -458,14 +460,14 @@ JpgInput::read_native_scanline(int subimage, int miplevel, int y, int z,
         // we'll have to convert.
         m_cmyk_buf.resize(m_spec.width * 4);
         readdata = &m_cmyk_buf[0];
-        ASSERT(m_spec.nchannels == 3);
+        OIIO_DASSERT(m_spec.nchannels == 3);
     }
 
     for (; m_next_scanline <= y; ++m_next_scanline) {
         // Keep reading until we've read the scanline we really need
         if (jpeg_read_scanlines(&m_cinfo, (JSAMPLE**)&readdata, 1) != 1
             || m_fatalerr) {
-            error("JPEG failed scanline read (\"%s\")", filename().c_str());
+            errorf("JPEG failed scanline read (\"%s\")", filename());
             return false;
         }
     }
@@ -482,9 +484,11 @@ JpgInput::read_native_scanline(int subimage, int miplevel, int y, int z,
 bool
 JpgInput::close()
 {
-    if (m_fd != NULL) {
+    if (m_io) {
         // unnecessary?  jpeg_abort_decompress (&m_cinfo);
-        jpeg_destroy_decompress(&m_cinfo);
+        if (m_decomp_create)
+            jpeg_destroy_decompress(&m_cinfo);
+        m_decomp_create = false;
         close_file();
     }
     init();  // Reset to initial state
