@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cmath>
 #include <cstdio>
@@ -194,7 +168,8 @@ ImageInput::read_scanline(int y, int z, TypeDesc format, void* data,
 
     // Complex case -- either changing data type or stride
     int scanline_values = m_spec.width * m_spec.nchannels;
-    unsigned char* buf  = (unsigned char*)alloca(m_spec.scanline_bytes(true));
+    unsigned char* buf  = OIIO_ALLOCA(unsigned char,
+                                     m_spec.scanline_bytes(true));
     bool ok = read_native_scanline(current_subimage(), current_miplevel(), y, z,
                                    buf);
     if (!ok)
@@ -209,7 +184,7 @@ ImageInput::read_scanline(int y, int z, TypeDesc format, void* data,
                                         xstride, AutoStride, AutoStride);
     } else {
         // Per-channel formats -- have to convert/copy channels individually
-        ASSERT(m_spec.channelformats.size() == (size_t)m_spec.nchannels);
+        OIIO_DASSERT(m_spec.channelformats.size() == (size_t)m_spec.nchannels);
         size_t offset = 0;
         for (int c = 0; ok && c < m_spec.nchannels; ++c) {
             TypeDesc chanformat = m_spec.channelformats[c];
@@ -397,11 +372,11 @@ ImageInput::read_native_scanlines(int subimage, int miplevel, int ybegin,
     // copies the appropriate subset.
     size_t prefix_bytes   = spec.pixel_bytes(0, chbegin, true);
     size_t subset_bytes   = spec.pixel_bytes(chbegin, chend, true);
-    size_t subset_ystride = spec.width * subset_bytes;
+    size_t subset_ystride = size_t(spec.width) * subset_bytes;
 
     // Read all channels of the scanlines into a temp buffer.
     size_t native_pixel_bytes = spec.pixel_bytes(true);
-    size_t native_ystride     = spec.width * native_pixel_bytes;
+    size_t native_ystride     = size_t(spec.width) * native_pixel_bytes;
     std::unique_ptr<char[]> buf(new char[native_ystride * (yend - ybegin)]);
     yend    = std::min(yend, spec.y + spec.height);
     bool ok = read_native_scanlines(subimage, miplevel, ybegin, yend, z,
@@ -482,7 +457,7 @@ ImageInput::read_tile(int x, int y, int z, TypeDesc format, void* data,
                                         xstride, ystride, zstride);
     } else {
         // Per-channel formats -- have to convert/copy channels individually
-        ASSERT(m_spec.channelformats.size() == (size_t)m_spec.nchannels);
+        OIIO_DASSERT(m_spec.channelformats.size() == (size_t)m_spec.nchannels);
         size_t offset = 0;
         for (int c = 0; c < m_spec.nchannels; ++c) {
             TypeDesc chanformat = m_spec.channelformats[c];
@@ -978,7 +953,7 @@ ImageInput::read_native_deep_image(int subimage, int miplevel,
         return false;
 
     if (spec.depth > 1) {
-        error(
+        errorf(
             "read_native_deep_image is not supported for volume (3D) images.");
         return false;
         // FIXME? - not implementing 3D deep images for now.  The only
@@ -1024,8 +999,9 @@ void
 ImageInput::append_error(const std::string& message) const
 {
     lock_guard lock(m_mutex);
-    ASSERT(m_errmessage.size() < 1024 * 1024 * 16
-           && "Accumulated error messages > 16MB. Try checking return codes!");
+    OIIO_DASSERT(
+        m_errmessage.size() < 1024 * 1024 * 16
+        && "Accumulated error messages > 16MB. Try checking return codes!");
     if (m_errmessage.size())
         m_errmessage += '\n';
     m_errmessage += message;
