@@ -1,32 +1,6 @@
-/*
-  Copyright 2011 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <iostream>
 #include <vector>
@@ -45,6 +19,9 @@
 
 using namespace OIIO;
 
+// Aid for things that are too short to benchmark accurately
+#define REP10(x) x, x, x, x, x, x, x, x, x, x
+
 static int iterations = 1000000;
 static int ntrials    = 5;
 static bool verbose   = false;
@@ -144,6 +121,10 @@ test_int_helpers()
     OIIO_CHECK_EQUAL(round_to_multiple_of_pow2(size_t(3), size_t(4)), 4);
     OIIO_CHECK_EQUAL(round_to_multiple_of_pow2(size_t(4), size_t(4)), 4);
     OIIO_CHECK_EQUAL(round_to_multiple_of_pow2(size_t(5), size_t(4)), 8);
+
+    OIIO_CHECK_EQUAL(rotl(uint32_t(0x12345678), 4), uint32_t(0x23456781));
+    OIIO_CHECK_EQUAL(rotl(uint64_t(0x123456789abcdef0ULL), 4),
+                     uint64_t(0x23456789abcdef01ULL));
 }
 
 
@@ -161,9 +142,12 @@ test_math_functions()
     OIIO_CHECK_EQUAL(ifloor(0.999f), 0);
     OIIO_CHECK_EQUAL(ifloor(1.0f), 1);
     OIIO_CHECK_EQUAL(ifloor(1.001f), 1);
-    float fval = 1.1;
+    float fval = 1.1f;
     clobber(fval);
     bench("ifloor", [&]() { return DoNotOptimize(ifloor(fval)); });
+    fval = -1.1f;
+    clobber(fval);
+    bench("ifloor (neg)", [&]() { return DoNotOptimize(ifloor(fval)); });
 
     int ival;
     OIIO_CHECK_EQUAL_APPROX(floorfrac(0.0f, &ival), 0.0f);
@@ -187,6 +171,112 @@ test_math_functions()
     OIIO_CHECK_EQUAL(sign(3.1f), 1.0f);
     OIIO_CHECK_EQUAL(sign(-3.1f), -1.0f);
     OIIO_CHECK_EQUAL(sign(0.0f), 0.0f);
+
+    {
+        OIIO_CHECK_EQUAL(fast_neg(1.5f), -1.5f);
+        OIIO_CHECK_EQUAL(fast_neg(-1.5f), 1.5f);
+        OIIO_CHECK_EQUAL(fast_neg(0.0f), 0.0f);
+        OIIO_CHECK_EQUAL(fast_neg(-0.0f), 0.0f);
+        float x = -3.5f;
+        clobber(x);
+        bench("-float x10", [&]() { return REP10(DoNotOptimize(-x)); });
+        bench("fast_neg(float) x10",
+              [&]() { return REP10(DoNotOptimize(fast_neg(x))); });
+    }
+
+    {
+        float a = 2.5f, b = 1.5f, c = 8.5f;
+        clobber(a);
+        clobber(b);
+        clobber(c);
+        bench("madd fake a*b+c", [&]() { return DoNotOptimize(a * b + c); });
+        bench("madd(a,b,c)",
+              [&]() { return DoNotOptimize(OIIO::madd(a, b, c)); });
+        bench("std::fma(a,b,c)",
+              [&]() { return DoNotOptimize(std::fma(a, b, c)); });
+    }
+    {
+        float a = 2.5f, b = 1.5f, c = 8.5f;
+        OIIO_CHECK_EQUAL(clamp(2.5f, 1.5f, 8.5f), 2.5f);
+        OIIO_CHECK_EQUAL(clamp(1.5f, 2.5f, 8.5f), 2.5f);
+        OIIO_CHECK_EQUAL(clamp(8.5f, 1.5f, 2.5f), 2.5f);
+        clobber(a);
+        clobber(b);
+        clobber(c);
+        bench("clamp(f,f,f) middle",
+              [&]() { return DoNotOptimize(clamp(a, b, c)); });
+        bench("clamp(f,f,f) low",
+              [&]() { return DoNotOptimize(clamp(b, a, c)); });
+        bench("clamp(f,f,f) high",
+              [&]() { return DoNotOptimize(clamp(c, b, a)); });
+    }
+
+    {
+        float x = 1.3f, y = 2.5f;
+        clobber(x, y);
+        bench("std::cos", [&]() { return DoNotOptimize(std::cos(x)); });
+        bench("fast_cos", [&]() { return DoNotOptimize(fast_cos(x)); });
+        bench("fast_cospi", [&]() { return DoNotOptimize(fast_cospi(x)); });
+        bench("std::sin", [&]() { return DoNotOptimize(std::sin(x)); });
+        bench("fast_sin", [&]() { return DoNotOptimize(fast_sin(x)); });
+        bench("fast_sinpi", [&]() { return DoNotOptimize(fast_sinpi(x)); });
+        bench("std::tan", [&]() { return DoNotOptimize(std::tan(x)); });
+        bench("fast_tan", [&]() { return DoNotOptimize(fast_tan(x)); });
+        bench("std::acos", [&]() { return DoNotOptimize(std::acos(x)); });
+        bench("fast_acos", [&]() { return DoNotOptimize(fast_acos(x)); });
+        bench("std::asin", [&]() { return DoNotOptimize(std::asin(x)); });
+        bench("fast_asin", [&]() { return DoNotOptimize(fast_asin(x)); });
+        bench("std::atan2", [&]() { return DoNotOptimize(std::atan2(y, x)); });
+        bench("fast_atan2", [&]() { return DoNotOptimize(fast_atan2(y, x)); });
+
+        bench("std::log2", [&]() { return DoNotOptimize(std::log2(x)); });
+        bench("fast_log2", [&]() { return DoNotOptimize(fast_log2(x)); });
+        bench("std::log", [&]() { return DoNotOptimize(std::log(x)); });
+        bench("fast_log", [&]() { return DoNotOptimize(fast_log(x)); });
+        bench("std::log10", [&]() { return DoNotOptimize(std::log10(x)); });
+        bench("fast_log10", [&]() { return DoNotOptimize(fast_log10(x)); });
+        bench("std::exp", [&]() { return DoNotOptimize(std::exp(x)); });
+        bench("fast_exp", [&]() { return DoNotOptimize(fast_exp(x)); });
+        bench("fast_correct_exp",
+              [&]() { return DoNotOptimize(fast_correct_exp(x)); });
+        bench("std::exp2", [&]() { return DoNotOptimize(std::exp2(x)); });
+        bench("fast_exp2", [&]() { return DoNotOptimize(fast_exp2(x)); });
+
+        OIIO_CHECK_EQUAL(safe_fmod(5.0f, 2.5f), 0.0f);
+        OIIO_CHECK_EQUAL(safe_fmod(-5.0f, 2.5f), 0.0f);
+        OIIO_CHECK_EQUAL(safe_fmod(-5.0f, -2.5f), 0.0f);
+        OIIO_CHECK_EQUAL(safe_fmod(5.5f, 2.5f), 0.5f);
+        OIIO_CHECK_EQUAL(safe_fmod(-5.5f, 2.5f), -0.5f);
+        OIIO_CHECK_EQUAL(safe_fmod(-5.5f, -2.5f), -0.5f);
+        OIIO_CHECK_EQUAL(safe_fmod(5.5f, 0.0f), 0.0f);
+        bench("std::fmod", [&]() { return DoNotOptimize(std::fmod(y, x)); });
+        bench("safe_fmod", [&]() { return DoNotOptimize(safe_fmod(y, x)); });
+    }
+
+    {
+        OIIO_CHECK_EQUAL(fast_rint(0.0f), 0);
+        OIIO_CHECK_EQUAL(fast_rint(-1.0f), -1);
+        OIIO_CHECK_EQUAL(fast_rint(-1.2f), -1);
+        OIIO_CHECK_EQUAL(fast_rint(-0.8f), -1);
+        OIIO_CHECK_EQUAL(fast_rint(-1.49f), -1);
+        OIIO_CHECK_EQUAL(fast_rint(-1.50f), -2);
+        OIIO_CHECK_EQUAL(fast_rint(-1.51f), -2);
+        OIIO_CHECK_EQUAL(fast_rint(1.0f), 1);
+        OIIO_CHECK_EQUAL(fast_rint(1.2f), 1);
+        OIIO_CHECK_EQUAL(fast_rint(0.8f), 1);
+        OIIO_CHECK_EQUAL(fast_rint(1.49f), 1);
+        OIIO_CHECK_EQUAL(fast_rint(1.50f), 2);
+        OIIO_CHECK_EQUAL(fast_rint(1.51f), 2);
+        float a = 1.5f;
+        clobber(a);
+        bench("fast_rint", [&]() { return DoNotOptimize(fast_rint(a)); });
+        bench("std::lrint", [&]() { return DoNotOptimize(std::lrint(a)); });
+        bench("int(std::rint)",
+              [&]() { return DoNotOptimize(static_cast<int>(std::rint(a))); });
+        bench("int(x+copysignf(0.5f,x))", [&]() {
+            return DoNotOptimize(static_cast<int>(a + copysignf(0.5f, a)));
+        });
+    }
 }
 
 
@@ -278,6 +368,67 @@ test_bit_range_convert()
 
 
 
+void
+test_packbits()
+{
+    std::cout << "test_convert_pack_bits\n";
+
+    {
+        unsigned char foo[3] = { 0, 0, 0 };
+        unsigned char* fp    = foo;
+        int fpf              = 0;
+        bitstring_add_n_bits(fp, fpf, 1, 4);
+        bitstring_add_n_bits(fp, fpf, 2, 8);
+        bitstring_add_n_bits(fp, fpf, 0xffff, 10);
+        // result should be 0x10 0x2f 0xfc
+        Strutil::printf("  bitstring_add_n_bits results %02x %02x %02x\n",
+                        foo[0], foo[1], foo[2]);
+        OIIO_CHECK_EQUAL(foo[0], 0x10);
+        OIIO_CHECK_EQUAL(foo[1], 0x2f);
+        OIIO_CHECK_EQUAL(foo[2], 0xfc);
+    }
+    {
+        unsigned char foo[4] = { 0, 0, 0, 0 };
+        unsigned char* fp    = foo;
+        int fpf              = 0;
+        bitstring_add_n_bits(fp, fpf, 1023, 10);
+        bitstring_add_n_bits(fp, fpf, 0, 10);
+        bitstring_add_n_bits(fp, fpf, 1023, 10);
+        // result should be 1111111111 0000000000 1111111111 00
+        //                     f   f    c   0   0    f   f    c
+        Strutil::printf("  bitstring_add_n_bits results %02x %02x %02x %02x\n",
+                        foo[0], foo[1], foo[2], foo[3]);
+        OIIO_CHECK_EQUAL(foo[0], 0xff);
+        OIIO_CHECK_EQUAL(foo[1], 0xc0);
+        OIIO_CHECK_EQUAL(foo[2], 0x0f);
+        OIIO_CHECK_EQUAL(foo[3], 0xfc);
+    }
+
+    const uint16_t u16vals[8] = { 1, 1, 1, 1, 1, 1, 1, 1 };
+    uint16_t u10[5]           = { 255, 255, 255, 255, 255 };
+    Strutil::printf(
+        " in 16 bit values: %04x %04x %04x %04x %04x %04x %04x %04x\n",
+        u16vals[0], u16vals[1], u16vals[2], u16vals[3], u16vals[4], u16vals[5],
+        u16vals[6], u16vals[7]);
+    bit_pack(cspan<uint16_t>(u16vals, 8), u10, 10);
+    Strutil::printf(
+        " packed to 10 bits, as 16 bit values: %04x %04x %04x %04x %04x\n",
+        u10[0], u10[1], u10[2], u10[3], u10[4]);
+    uint16_t u16[8];
+    bit_unpack(8, (const unsigned char*)u10, 10, u16);
+    Strutil::printf(
+        " unpacked back to 16 bits: %04x %04x %04x %04x %04x %04x %04x %04x\n",
+        u16[0], u16[1], u16[2], u16[3], u16[4], u16[5], u16[6], u16[7]);
+    // Before: 00000000 00000001  00000000 00000001  00000000 00000001...
+    // After:  00000000 01000000  00010000 00000100  00000001 00000000  01000000 00010000  00000100 00000001
+    //       =  00 40  10 04  01 00  40 10  04 01
+    // as little endian 16 bit:  4000 0410 0001 1040 0104
+    for (size_t i = 0; i < 8; ++i)
+        OIIO_CHECK_EQUAL(u16vals[i], u16[i]);
+}
+
+
+
 static void
 test_interpolate_linear()
 {
@@ -439,6 +590,7 @@ main(int argc, char* argv[])
     //    test_convert_type<float,unsigned short> ();
 
     test_bit_range_convert();
+    test_packbits();
 
     test_interpolate_linear();
 
