@@ -1,38 +1,14 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
 #include <iostream>
+#include <mutex>
+#include <signal.h>
 #include <string>
 #include <thread>
 
@@ -82,6 +58,14 @@
 #include <OpenImageIO/strutil.h>
 #include <OpenImageIO/sysutil.h>
 
+#include <boost/version.hpp>
+#if BOOST_VERSION >= 106500
+#    ifndef _GNU_SOURCE
+#        define _GNU_SOURCE
+#    endif
+#    include <boost/stacktrace.hpp>
+#endif
+
 // clang 7.0 (rc2) has errors when including boost thread!
 // The only thin we're using there is boost::physical_concurrency.
 #if !(OIIO_CLANG_VERSION >= 7)
@@ -147,7 +131,7 @@ Sysutil::memory_used(bool resident)
     return 0;  // Punt
 #else
     // No idea what platform this is
-    ASSERT(0 && "Need to implement Sysutil::memory_used on this platform");
+    OIIO_ASSERT(0 && "Need to implement Sysutil::memory_used on this platform");
     return 0;  // Punt
 #endif
 }
@@ -212,7 +196,8 @@ Sysutil::physical_memory()
 
 #else
     // No idea what platform this is
-    ASSERT(0 && "Need to implement Sysutil::physical_memory on this platform");
+    OIIO_ASSERT(
+        0 && "Need to implement Sysutil::physical_memory on this platform");
     return 0;  // Punt
 #endif
 }
@@ -248,7 +233,7 @@ Sysutil::this_program_path()
     unsigned int size = sizeof(filename);
     int r             = readlink("/proc/self/exe", filename, size);
     // user won't get the right answer if the filename is too long to store
-    ASSERT(r < int(size));
+    OIIO_ASSERT(r < int(size));
     if (r > 0)
         filename[r] = 0;  // readlink does not fill in the 0 byte
 #elif defined(__APPLE__)
@@ -538,7 +523,7 @@ Sysutil::put_in_background(int, char*[])
     return true;
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
     return true;
 #endif
 
@@ -582,6 +567,8 @@ Sysutil::max_open_files()
     return size_t(-1);  // Couldn't figure out, so return effectively infinity
 }
 
+
+
 void*
 aligned_malloc(std::size_t size, std::size_t align)
 {
@@ -593,6 +580,8 @@ aligned_malloc(std::size_t size, std::size_t align)
 #endif
 }
 
+
+
 void
 aligned_free(void* ptr)
 {
@@ -604,4 +593,60 @@ aligned_free(void* ptr)
 }
 
 
+
+std::string
+Sysutil::stacktrace()
+{
+#if BOOST_VERSION >= 106500
+    std::stringstream out;
+    out << boost::stacktrace::stacktrace();
+    return out.str();
+#else
+    return "";
+#endif
+}
+
+
+
+#if BOOST_VERSION >= 106500
+
+static std::string stacktrace_filename;
+static std::mutex stacktrace_filename_mutex;
+
+static void
+stacktrace_signal_handler(int signum)
+{
+    ::signal(signum, SIG_DFL);
+    if (!stacktrace_filename.empty()) {
+        if (stacktrace_filename == "stdout")
+            std::cout << Sysutil::stacktrace();
+        else if (stacktrace_filename == "stderr")
+            std::cerr << Sysutil::stacktrace();
+        else {
+#    if BOOST_VERSION >= 106500
+            boost::stacktrace::safe_dump_to(stacktrace_filename.c_str());
+#    endif
+        }
+    }
+    ::raise(SIGABRT);
+}
+
+#endif
+
+
+
+bool
+Sysutil::setup_crash_stacktrace(string_view filename)
+{
+#if BOOST_VERSION >= 106500
+    std::lock_guard<std::mutex> lock(stacktrace_filename_mutex);
+    stacktrace_filename = filename;
+    ::signal(SIGSEGV, &stacktrace_signal_handler);
+    ::signal(SIGABRT, &stacktrace_signal_handler);
+    return true;
+#endif
+    return false;
+}
+
+
 OIIO_NAMESPACE_END
