@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cerrno>
 #include <cmath>
@@ -37,11 +11,28 @@
 #include <map>
 #include <memory>
 
+#include <OpenImageIO/platform.h>
+
 #include <OpenEXR/ImfChannelList.h>
 #include <OpenEXR/ImfEnvmap.h>
 #include <OpenEXR/ImfOutputFile.h>
 #include <OpenEXR/ImfTiledOutputFile.h>
 
+#ifdef OPENEXR_VERSION_MAJOR
+#    define OPENEXR_CODED_VERSION                                              \
+        (OPENEXR_VERSION_MAJOR * 10000 + OPENEXR_VERSION_MINOR * 100           \
+         + OPENEXR_VERSION_PATCH)
+#else
+#    define OPENEXR_CODED_VERSION 20000
+#endif
+
+#if OPENEXR_CODED_VERSION >= 20400                                             \
+    || __has_include(<OpenEXR/ImfFloatVectorAttribute.h>)
+#    define OPENEXR_HAS_FLOATVECTOR 1
+#else
+#    define OPENEXR_HAS_FLOATVECTOR 0
+#endif
+
 // The way that OpenEXR uses dynamic casting for attributes requires
 // temporarily suspending "hidden" symbol visibility mode.
 #ifdef __GNUC__
@@ -54,6 +45,9 @@
 #include <OpenEXR/ImfCompressionAttribute.h>
 #include <OpenEXR/ImfEnvmapAttribute.h>
 #include <OpenEXR/ImfFloatAttribute.h>
+#if OPENEXR_HAS_FLOATVECTOR
+#    include <OpenEXR/ImfFloatVectorAttribute.h>
+#endif
 #include <OpenEXR/ImfIntAttribute.h>
 #include <OpenEXR/ImfKeyCodeAttribute.h>
 #include <OpenEXR/ImfMatrixAttribute.h>
@@ -367,18 +361,18 @@ OpenEXROutput::open(const std::string& name, const ImageSpec& userspec,
                                         m_headers[m_subimage]));
             }
         } catch (const std::exception& e) {
-            error("OpenEXR exception: %s", e.what());
+            errorf("Could not open \"%s\" (%s)", name, e.what());
             m_output_scanline = NULL;
             m_output_tiled    = NULL;
             return false;
         } catch (...) {  // catch-all for edge cases or compiler bugs
-            error("OpenEXR exception: unknown");
+            errorf("Could not open \"%s\" (unknown exception)", name);
             m_output_scanline = NULL;
             m_output_tiled    = NULL;
             return false;
         }
         if (!m_output_scanline && !m_output_tiled) {
-            error("Unknown error opening EXR file");
+            errorf("Unknown error opening EXR file");
             return false;
         }
 
@@ -389,13 +383,13 @@ OpenEXROutput::open(const std::string& name, const ImageSpec& userspec,
         // OpenEXR 2.x supports subimages, but we only allow it to use the
         // open(name,subimages,specs[]) variety.
         if (m_subimagespecs.size() == 0 || !m_output_multipart) {
-            error("%s not opened properly for subimages", format_name());
+            errorf("%s not opened properly for subimages", format_name());
             return false;
         }
         // Move on to next subimage
         ++m_subimage;
         if (m_subimage >= m_nsubimages) {
-            error("More subimages than originally declared.");
+            errorf("More subimages than originally declared.");
             return false;
         }
         // Close the current subimage, open the next one
@@ -415,19 +409,19 @@ OpenEXROutput::open(const std::string& name, const ImageSpec& userspec,
                     new Imf::DeepScanLineOutputPart(*m_output_multipart,
                                                     m_subimage));
             } else {
-                error(
+                errorf(
                     "Called open with AppendSubimage mode, but no appropriate part is found. Application bug?");
                 return false;
             }
         } catch (const std::exception& e) {
-            error("OpenEXR exception: %s", e.what());
+            errorf("OpenEXR exception: %s", e.what());
             m_scanline_output_part.reset();
             m_tiled_output_part.reset();
             m_deep_scanline_output_part.reset();
             m_deep_tiled_output_part.reset();
             return false;
         } catch (...) {  // catch-all for edge cases or compiler bugs
-            error("OpenEXR exception: unknown exception");
+            errorf("OpenEXR exception: unknown exception");
             m_scanline_output_part.reset();
             m_tiled_output_part.reset();
             m_deep_scanline_output_part.reset();
@@ -442,7 +436,7 @@ OpenEXROutput::open(const std::string& name, const ImageSpec& userspec,
 
     if (mode == AppendMIPLevel) {
         if (!m_output_scanline && !m_output_tiled) {
-            error("Cannot append a MIP level if no file has been opened");
+            errorf("Cannot append a MIP level if no file has been opened");
             return false;
         }
         if (m_spec.tile_width && m_levelmode != Imf::ONE_LEVEL) {
@@ -451,7 +445,7 @@ OpenEXROutput::open(const std::string& name, const ImageSpec& userspec,
             // tile sizes.
             if (userspec.tile_width != m_spec.tile_width
                 || userspec.tile_height != m_spec.tile_height) {
-                error(
+                errorf(
                     "OpenEXR tiles must have the same size on all MIPmap levels");
                 return false;
             }
@@ -463,12 +457,12 @@ OpenEXROutput::open(const std::string& name, const ImageSpec& userspec,
             ++m_miplevel;
             return true;
         } else {
-            error("Cannot add MIP level to a non-MIPmapped file");
+            errorf("Cannot add MIP level to a non-MIPmapped file");
             return false;
         }
     }
 
-    error("Unknown open mode %d", int(mode));
+    errorf("Unknown open mode %d", int(mode));
     return false;
 }
 
@@ -479,7 +473,7 @@ OpenEXROutput::open(const std::string& name, int subimages,
                     const ImageSpec* specs)
 {
     if (subimages < 1) {
-        error("OpenEXR does not support %d subimages.", subimages);
+        errorf("OpenEXR does not support %d subimages.", subimages);
         return false;
     }
 
@@ -505,7 +499,7 @@ OpenEXROutput::open(const std::string& name, int subimages,
             return false;
         deep |= m_subimagespecs[s].deep;
         if (m_subimagespecs[s].deep != m_subimagespecs[0].deep) {
-            error(
+            errorf(
                 "OpenEXR does not support mixed deep/nondeep multi-part image files");
             return false;
         }
@@ -534,11 +528,11 @@ OpenEXROutput::open(const std::string& name, int subimages,
                                                               subimages));
     } catch (const std::exception& e) {
         m_output_stream.reset();
-        error("OpenEXR exception: %s", e.what());
+        errorf("OpenEXR exception: %s", e.what());
         return false;
     } catch (...) {  // catch-all for edge cases or compiler bugs
         m_output_stream.reset();
-        error("OpenEXR exception: unknown exception");
+        errorf("OpenEXR exception: unknown exception");
         return false;
     }
     try {
@@ -560,7 +554,7 @@ OpenEXROutput::open(const std::string& name, int subimages,
             }
         }
     } catch (const std::exception& e) {
-        error("OpenEXR exception: %s", e.what());
+        errorf("OpenEXR exception: %s", e.what());
         m_output_stream.reset();
         m_scanline_output_part.reset();
         m_tiled_output_part.reset();
@@ -568,7 +562,7 @@ OpenEXROutput::open(const std::string& name, int subimages,
         m_deep_tiled_output_part.reset();
         return false;
     } catch (...) {  // catch-all for edge cases or compiler bugs
-        error("OpenEXR exception: unknown exception");
+        errorf("OpenEXR exception: unknown exception");
         m_output_stream.reset();
         m_scanline_output_part.reset();
         m_tiled_output_part.reset();
@@ -601,7 +595,7 @@ OpenEXROutput::compute_pixeltypes(const ImageSpec& spec)
         }
         m_pixeltype.push_back(ptype);
     }
-    ASSERT(m_pixeltype.size() == size_t(spec.nchannels));
+    OIIO_ASSERT(m_pixeltype.size() == size_t(spec.nchannels));
 }
 
 
@@ -611,14 +605,14 @@ OpenEXROutput::spec_to_header(ImageSpec& spec, int subimage,
                               Imf::Header& header)
 {
     if (spec.width < 1 || spec.height < 1) {
-        error("Image resolution must be at least 1x1, you asked for %d x %d",
-              spec.width, spec.height);
+        errorf("Image resolution must be at least 1x1, you asked for %d x %d",
+               spec.width, spec.height);
         return false;
     }
     if (spec.depth < 1)
         spec.depth = 1;
     if (spec.depth > 1) {
-        error("%s does not support volume images (depth > 1)", format_name());
+        errorf("%s does not support volume images (depth > 1)", format_name());
         return false;
     }
 
@@ -745,10 +739,8 @@ OpenEXROutput::spec_to_header(ImageSpec& spec, int subimage,
                                  Imf::LevelRoundingMode(m_roundingmode)));
 
     // Deal with all other params
-    for (size_t p = 0; p < spec.extra_attribs.size(); ++p)
-        put_parameter(spec.extra_attribs[p].name().string(),
-                      spec.extra_attribs[p].type(),
-                      spec.extra_attribs[p].data(), header);
+    for (const auto& p : spec.extra_attribs)
+        put_parameter(p.name().string(), p.type(), p.data(), header);
 
     // Multi-part EXR files required to have a name. Make one up if not
     // supplied.
@@ -841,8 +833,7 @@ static ExrMeta exr_meta_translation[] = {
     // user or from a file we read.
     ExrMeta("YResolution"), ExrMeta("planarconfig"), ExrMeta("type"),
     ExrMeta("tiles"), ExrMeta("version"), ExrMeta("chunkCount"),
-    ExrMeta("maxSamplesPerPixel"),
-    ExrMeta()  // empty name signifies end of list
+    ExrMeta("maxSamplesPerPixel"), ExrMeta("openexr:roundingmode")
 };
 
 
@@ -857,8 +848,7 @@ OpenEXROutput::put_parameter(const std::string& name, TypeDesc type,
     std::string xname = name;
     TypeDesc exrtype  = TypeUnknown;
 
-    for (int i = 0; exr_meta_translation[i].oiioname; ++i) {
-        const ExrMeta& e(exr_meta_translation[i]);
+    for (const auto& e : exr_meta_translation) {
         if (Strutil::iequals(xname, e.oiioname)
             || (e.exrname && Strutil::iequals(xname, e.exrname))) {
             xname   = std::string(e.exrname ? e.exrname : "");
@@ -1230,13 +1220,20 @@ OpenEXROutput::put_parameter(const std::string& name, TypeDesc type,
             }
             // String Vector
             if (type.basetype == TypeDesc::STRING) {
-                Imf::StringVector v;
-                for (int i = 0; i < type.arraylen; i++) {
-                    v.emplace_back(((const char**)data)[i]);
-                }
+                Imf::StringVector v((const char**)data,
+                                    (const char**)data + type.basevalues());
                 header.insert(xname.c_str(), Imf::StringVectorAttribute(v));
                 return true;
             }
+#if OPENEXR_HAS_FLOATVECTOR
+            // float Vector -- only supported in OpenEXR >= 2.2
+            if (type.basetype == TypeDesc::FLOAT) {
+                Imf::FloatVector v((const float*)data,
+                                   (const float*)data + type.basevalues());
+                header.insert(xname.c_str(), Imf::FloatVectorAttribute(v));
+                return true;
+            }
+#endif
         }
     } catch (const std::exception& e) {
         OIIO::debug("Caught OpenEXR exception: %s\n", e.what());
@@ -1302,7 +1299,7 @@ OpenEXROutput::write_scanline(int y, int z, TypeDesc format, const void* data,
                               stride_t xstride)
 {
     if (!(m_output_scanline || m_scanline_output_part)) {
-        error("called OpenEXROutput::write_scanline without an open file");
+        errorf("called OpenEXROutput::write_scanline without an open file");
         return false;
     }
 
@@ -1338,14 +1335,14 @@ OpenEXROutput::write_scanline(int y, int z, TypeDesc format, const void* data,
             m_scanline_output_part->setFrameBuffer(frameBuffer);
             m_scanline_output_part->writePixels(1);
         } else {
-            error("Attempt to write scanline to a non-scanline file.");
+            errorf("Attempt to write scanline to a non-scanline file.");
             return false;
         }
     } catch (const std::exception& e) {
-        error("Failed OpenEXR write: %s", e.what());
+        errorf("Failed OpenEXR write: %s", e.what());
         return false;
     } catch (...) {  // catch-all for edge cases or compiler bugs
-        error("Failed OpenEXR write: unknown exception");
+        errorf("Failed OpenEXR write: unknown exception");
         return false;
     }
 
@@ -1362,7 +1359,7 @@ OpenEXROutput::write_scanlines(int ybegin, int yend, int z, TypeDesc format,
                                stride_t ystride)
 {
     if (!(m_output_scanline || m_scanline_output_part)) {
-        error("called OpenEXROutput::write_scanlines without an open file");
+        errorf("called OpenEXROutput::write_scanlines without an open file");
         return false;
     }
 
@@ -1412,14 +1409,14 @@ OpenEXROutput::write_scanlines(int ybegin, int yend, int z, TypeDesc format,
                 m_scanline_output_part->setFrameBuffer(frameBuffer);
                 m_scanline_output_part->writePixels(nscanlines);
             } else {
-                error("Attempt to write scanlines to a non-scanline file.");
+                errorf("Attempt to write scanlines to a non-scanline file.");
                 return false;
             }
         } catch (const std::exception& e) {
-            error("Failed OpenEXR write: %s", e.what());
+            errorf("Failed OpenEXR write: %s", e.what());
             return false;
         } catch (...) {  // catch-all for edge cases or compiler bugs
-            error("Failed OpenEXR write: unknown exception");
+            errorf("Failed OpenEXR write: unknown exception");
             return false;
         }
 
@@ -1466,11 +1463,11 @@ OpenEXROutput::write_tiles(int xbegin, int xend, int ybegin, int yend,
     //    std::cerr << "exr::write_tiles " << xbegin << ' ' << xend
     //              << ' ' << ybegin << ' ' << yend << "\n";
     if (!(m_output_tiled || m_tiled_output_part)) {
-        error("called OpenEXROutput::write_tiles without an open file");
+        errorf("called OpenEXROutput::write_tiles without an open file");
         return false;
     }
     if (!m_spec.valid_tile_range(xbegin, xend, ybegin, yend, zbegin, zend)) {
-        error("called OpenEXROutput::write_tiles with an invalid tile range");
+        errorf("called OpenEXROutput::write_tiles with an invalid tile range");
         return false;
     }
 
@@ -1499,8 +1496,8 @@ OpenEXROutput::write_tiles(int xbegin, int xend, int ybegin, int yend,
     int nytiles = (yend - ybegin + m_spec.tile_height - 1) / m_spec.tile_height;
 
     std::vector<char> padded;
-    int width           = nxtiles * m_spec.tile_width;
-    int height          = nytiles * m_spec.tile_height;
+    stride_t width      = nxtiles * m_spec.tile_width;
+    stride_t height     = nytiles * m_spec.tile_height;
     stride_t widthbytes = width * pixelbytes;
     if (width != (xend - xbegin) || height != (yend - ybegin)) {
         // If the image region is not an even multiple of the tile size,
@@ -1540,14 +1537,14 @@ OpenEXROutput::write_tiles(int xbegin, int xend, int ybegin, int yend,
                                             firstytile + nytiles - 1,
                                             m_miplevel, m_miplevel);
         } else {
-            error("Attempt to write tiles for a non-tiled file.");
+            errorf("Attempt to write tiles for a non-tiled file.");
             return false;
         }
     } catch (const std::exception& e) {
-        error("Failed OpenEXR write: %s", e.what());
+        errorf("Failed OpenEXR write: %s", e.what());
         return false;
     } catch (...) {  // catch-all for edge cases or compiler bugs
-        error("Failed OpenEXR write: unknown exception");
+        errorf("Failed OpenEXR write: unknown exception");
         return false;
     }
 
@@ -1561,13 +1558,13 @@ OpenEXROutput::write_deep_scanlines(int ybegin, int yend, int z,
                                     const DeepData& deepdata)
 {
     if (m_deep_scanline_output_part == NULL) {
-        error(
+        errorf(
             "called OpenEXROutput::write_deep_scanlines without an open file");
         return false;
     }
     if (m_spec.width * (yend - ybegin) != deepdata.pixels()
         || m_spec.nchannels != deepdata.channels()) {
-        error(
+        errorf(
             "called OpenEXROutput::write_deep_scanlines with non-matching DeepData size");
         return false;
     }
@@ -1600,10 +1597,10 @@ OpenEXROutput::write_deep_scanlines(int ybegin, int yend, int z,
         // Write the pixels
         m_deep_scanline_output_part->writePixels(yend - ybegin);
     } catch (const std::exception& e) {
-        error("Failed OpenEXR write: %s", e.what());
+        errorf("Failed OpenEXR write: %s", e.what());
         return false;
     } catch (...) {  // catch-all for edge cases or compiler bugs
-        error("Failed OpenEXR write: unknown exception");
+        errorf("Failed OpenEXR write: unknown exception");
         return false;
     }
 
@@ -1617,12 +1614,12 @@ OpenEXROutput::write_deep_tiles(int xbegin, int xend, int ybegin, int yend,
                                 int zbegin, int zend, const DeepData& deepdata)
 {
     if (m_deep_tiled_output_part == NULL) {
-        error("called OpenEXROutput::write_deep_tiles without an open file");
+        errorf("called OpenEXROutput::write_deep_tiles without an open file");
         return false;
     }
     if ((xend - xbegin) * (yend - ybegin) * (zend - zbegin) != deepdata.pixels()
         || m_spec.nchannels != deepdata.channels()) {
-        error(
+        errorf(
             "called OpenEXROutput::write_deep_tiles with non-matching DeepData size");
         return false;
     }
@@ -1667,10 +1664,10 @@ OpenEXROutput::write_deep_tiles(int xbegin, int xend, int ybegin, int yend,
                                              firstytile + ytiles - 1,
                                              m_miplevel, m_miplevel);
     } catch (const std::exception& e) {
-        error("Failed OpenEXR write: %s", e.what());
+        errorf("Failed OpenEXR write: %s", e.what());
         return false;
     } catch (...) {  // catch-all for edge cases or compiler bugs
-        error("Failed OpenEXR write: unknown exception");
+        errorf("Failed OpenEXR write: unknown exception");
         return false;
     }
 
