@@ -1,32 +1,6 @@
-/*
-  Copyright 2013 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <algorithm>
 #include <ctime> /* time_t, struct tm, gmtime */
@@ -40,8 +14,8 @@
 #include <OpenImageIO/sysutil.h>
 #include <OpenImageIO/tiffutils.h>
 
-#if OIIO_GNUC_VERSION >= 80000 || OIIO_CLANG_VERSION >= 50000
-// fix gcc8 warnings in libraw headers: use of auto_ptr
+#if OIIO_GNUC_VERSION || OIIO_CLANG_VERSION >= 50000
+// fix warnings in libraw headers: use of auto_ptr
 #    pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
 
@@ -379,24 +353,29 @@ RawInput::open_raw(bool unpack, const std::string& name,
 
     int ret;
     if ((ret = m_processor->open_file(name.c_str())) != LIBRAW_SUCCESS) {
-        error("Could not open file \"%s\", %s", m_filename,
-              libraw_strerror(ret));
+        errorf("Could not open file \"%s\", %s", m_filename,
+               libraw_strerror(ret));
         return false;
     }
 
-    ASSERT(!m_unpacked);
+    OIIO_ASSERT(!m_unpacked);
     if (unpack) {
         if ((ret = m_processor->unpack()) != LIBRAW_SUCCESS) {
-            error("Could not unpack \"%s\", %s", m_filename,
-                  libraw_strerror(ret));
+            errorf("Could not unpack \"%s\", %s", m_filename,
+                   libraw_strerror(ret));
             return false;
         }
     }
     m_processor->adjust_sizes_info_only();
 
+    // Process image at half size if "raw:half_size" is not 0
+    m_processor->imgdata.params.half_size
+        = config.get_int_attribute("raw:half_size", 0);
+    int div = m_processor->imgdata.params.half_size == 0 ? 1 : 2;
+
     // Set file information
-    m_spec = ImageSpec(m_processor->imgdata.sizes.iwidth,
-                       m_processor->imgdata.sizes.iheight,
+    m_spec = ImageSpec(m_processor->imgdata.sizes.iwidth / div,
+                       m_processor->imgdata.sizes.iheight / div,
                        3,  // LibRaw should only give us 3 channels
                        TypeDesc::UINT16);
     // Move the exif attribs we already read into the spec we care about
@@ -428,6 +407,24 @@ RawInput::open_raw(bool unpack, const std::string& name,
             m_processor->imgdata.params.aber[2] = p->get<double>(1);
         }
     }
+    // Set user white balance coefficients.
+    // Only has effect if "raw:use_camera_wb" is equal to 0,
+    // i.e. we are not using the camera white balance
+    {
+        auto p = config.find_attribute("raw:user_mul");
+        if (p && p->type() == TypeDesc(TypeDesc::FLOAT, 4)) {
+            m_processor->imgdata.params.user_mul[0] = p->get<float>(0);
+            m_processor->imgdata.params.user_mul[1] = p->get<float>(1);
+            m_processor->imgdata.params.user_mul[2] = p->get<float>(2);
+            m_processor->imgdata.params.user_mul[3] = p->get<float>(3);
+        }
+        if (p && p->type() == TypeDesc(TypeDesc::DOUBLE, 4)) {
+            m_processor->imgdata.params.user_mul[0] = p->get<double>(0);
+            m_processor->imgdata.params.user_mul[1] = p->get<double>(1);
+            m_processor->imgdata.params.user_mul[2] = p->get<double>(2);
+            m_processor->imgdata.params.user_mul[3] = p->get<double>(3);
+        }
+    }
 
     // Use embedded color profile. Values mean:
     // 0: do not use embedded color profile
@@ -503,7 +500,7 @@ RawInput::open_raw(bool unpack, const std::string& name,
     float exposure = config.get_float_attribute("raw:Exposure", -1.0f);
     if (exposure >= 0.0f) {
         if (exposure < 0.25f || exposure > 8.0f) {
-            error("raw:Exposure invalid value. range 0.25f - 8.0f");
+            errorf("raw:Exposure invalid value. range 0.25f - 8.0f");
             return false;
         }
         m_processor->imgdata.params.exp_correc
@@ -518,7 +515,7 @@ RawInput::open_raw(bool unpack, const std::string& name,
     int highlight_mode = config.get_int_attribute("raw:HighlightMode", 0);
     if (highlight_mode != 0) {
         if (highlight_mode < 0 || highlight_mode > 9) {
-            error("raw:HighlightMode invalid value. range 0-9");
+            errorf("raw:HighlightMode invalid value. range 0-9");
             return false;
         }
         m_processor->imgdata.params.highlight = highlight_mode;
@@ -562,8 +559,8 @@ RawInput::open_raw(bool unpack, const std::string& name,
             libraw_decoder_info_t decoder_info;
             m_processor->get_decoder_info(&decoder_info);
             if (!(decoder_info.decoder_flags & LIBRAW_DECODER_FLATFIELD)) {
-                error("Unable to extract unbayered data from file \"%s\"",
-                      name.c_str());
+                errorf("Unable to extract unbayered data from file \"%s\"",
+                       name);
                 return false;
             }
 
@@ -581,7 +578,7 @@ RawInput::open_raw(bool unpack, const std::string& name,
             m_spec.erase_attribute("raw:Colorspace");
             m_spec.erase_attribute("raw:Exposure");
         } else {
-            error("raw:Demosaic set to unknown value");
+            errorf("raw:Demosaic set to unknown value");
             return false;
         }
         // Set the attribute in the output spec
@@ -870,7 +867,7 @@ RawInput::get_makernotes_olympus()
     // unsigned     AFAreas[64];
     MAKERF(AFPointSelected);
     MAKERF(AFResult);
-    MAKERF(ImageStabilization);
+    // MAKERF(ImageStabilization);  Removed after 0.19
     MAKERF(ColorSpace);
 #endif
 #if LIBRAW_VERSION >= LIBRAW_MAKE_VERSION(0, 19, 0)
@@ -996,7 +993,7 @@ RawInput::get_makernotes_sony()
         MAKERF(ElectronicFrontCurtainShutter);
     MAKER(MeteringMode2, 0);
     add(m_make, "DateTime", mn.SonyDateTime);
-    MAKERF(TimeStamp);
+    // MAKERF(TimeStamp);  Removed after 0.19, is in 'other'
     MAKER(ShotNumberSincePowerUp, 0);
 #endif
 }
@@ -1133,23 +1130,23 @@ RawInput::process()
     if (!m_image) {
         int ret = m_processor->dcraw_process();
         if (ret != LIBRAW_SUCCESS) {
-            error("Processing image failed, %s", libraw_strerror(ret));
+            errorf("Processing image failed, %s", libraw_strerror(ret));
             return false;
         }
 
         m_image = m_processor->dcraw_make_mem_image(&ret);
         if (!m_image) {
-            error("LibRaw failed to create in memory image");
+            errorf("LibRaw failed to create in memory image");
             return false;
         }
 
         if (m_image->type != LIBRAW_IMAGE_BITMAP) {
-            error("LibRaw did not return expected image type");
+            errorf("LibRaw did not return expected image type");
             return false;
         }
 
         if (m_image->colors != 3) {
-            error("LibRaw did not return 3 channel image");
+            errorf("LibRaw did not return 3 channel image");
             return false;
         }
     }
