@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 
 from __future__ import print_function
+from __future__ import absolute_import
 import OpenImageIO as oiio
 
 
@@ -90,6 +91,9 @@ try:
     s.attribute ("foo_matrix", oiio.TypeDesc.TypeMatrix,
                  (1, 0, 0, 0, 0, 2, 0, 0, 0, 0, 1, 0, 1, 2, 3, 1))
     s.attribute ("smpte:TimeCode", oiio.TypeDesc.TypeTimeCode, (18356486, 4294967295))
+    s["delfoo_str"] =  "egg"
+    s["delfoo_int"] = 29
+    s["delfoo_float"] = 99.5
     print ("get_int_attribute('foo_int') retrieves", s.get_int_attribute ("foo_int"))
     print ("get_int_attribute('foo_int',21) with default retrieves", s.get_int_attribute ("foo_int", 21))
     print ("get_int_attribute('foo_no',23) retrieves", s.get_int_attribute ("foo_no", 23))
@@ -107,6 +111,9 @@ try:
     print ("getattribute('foo_matrix') retrieves", s.getattribute("foo_matrix"))
     print ("getattribute('foo_no') retrieves", s.getattribute("foo_no"))
     print ("getattribute('smpte:TimeCode') retrieves", s.getattribute("smpte:TimeCode"))
+    print ("s['delfoo_float'] =", s['delfoo_float'])
+    print ("s['delfoo_int'] =", s['delfoo_int'])
+    print ("s['delfoo_str'] =", s['delfoo_str'])
     print ()
 
     print ("extra_attribs size is", len(s.extra_attribs))
